function Y=svmlight(xtrain, ytrain, xtest, params)
% Wrapper for svmlight, version 6.02
% write matrices in sparse format to data file that can be used by svmlight.
% Columns are variables, rows are observations. 
%
% These steps are made: 
% 1. output matlab matrix to text file
% 2. format text file for svm (awk)
% 3. create classification model (svm_learn)
% 4. apply classification model (svm_classify)
%
% All files are written in the /tmp/ directory, if you are on windows you might 
% want to change that to the current directory ("."). Obviously you need awk 
% installed for this function to work. Assumes svm-light is installed in the svmlight 
% subdirectory (change path if necessary). 
%
% Example: 
% Y=svmlight(Xtrain, Ytrain, Xtest, '-t 0 -c 0.5 ');
% (if you set parameters for svmlight don't forget to include the learning options!)
%
% (c) Benjamin Auffarth, 2008
% modified by Debajyoti Ray, 2010
if nargin<4
  params='-t 0 -c 0.5 ';
end

training = [ytrain, xtrain];
test = xtest;

trainfile=sparse_write(training);
[s,w]=system(['svm_light/svm_learn ' params trainfile '.svm2 ' trainfile '.model']);
if s 
  disp('error in executing smv-light!');w,
  error('svm_learn not found or returned error');
end

testfile=sparse_write(test);
[s,w]=system(['svm_light/svm_classify -v 0  ' testfile '.svm2 ' trainfile '.model ' testfile '.dat']);
if s 
  disp('error in executing smv-light!');w,
  error('svm_classify not found or returned error');
end
Y=dlmread([testfile '.dat']);
end

function fname=sparse_write(M)
  [a,fname]=system('date +/tmp/_svm_%F_-%H:%M_%S%N');
  fname=fname(1:end-1); % get rid of newline character
  dlmwrite([fname '.svm1'],M,'delimiter',' '); 
  system(['awk -F" " ''{printf $1" "; for (i=2;i<=NF;i++) {printf i-1":"$i " "}; print ""}'' ' fname '.svm1 > ' fname '.svm2']);
end
